/*****************************************************************************
   xamixer.c - an Alsa based gtk mixer
   Written by Raistlinn (lansdoct@cs.alfred.edu)
   Copyright (C) 1998 by Christopher Lansdown
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

/*****************************************************************************/
/* Begin #include's */

#include "main.h"

/* End #include's */
/*****************************************************************************/

/*****************************************************************************/
/* Begin Global Variables */

extern GtkWidget *window;
extern Card *card; /* And array of the cards */
extern int cards; /* The number of cards in the system. */
extern Config config; /* The system config */

/* End Global Variables */
/*****************************************************************************/


GtkWidget *create_volume_control(char *name, 
				 int is_xpm,
				 float left, 
				 float right, 
				 Mixer *mixer, 
				 int channel, 
				 snd_mixer_channel_t *data, 
				 snd_mixer_channel_info_t *chan_info, 
				 Channel *schan,
				 GtkWidget *xpmparent,
				 GtkWidget *table)
{
  GtkWidget *box=NULL, *box1=NULL, *box2=NULL, *tmpbox=NULL;
  GtkWidget *rscale=NULL, *lscale=NULL;
  GtkObject *ladj=NULL, *radj=NULL, *madj=NULL; 
  GtkWidget *label=NULL;
  GtkWidget *lmute=NULL, *rmute=NULL, *mmute=NULL;
  GtkWidget *lrec=NULL, *rrec=NULL, *mrec=NULL;
  GtkWidget *button=NULL;
  GtkWidget *chan_xpm=NULL;
  GtkWidget *rmute_xpm=NULL, *runmute_xpm=NULL;
  GtkWidget *lmute_xpm=NULL, *lunmute_xpm=NULL;
  GtkWidget *mmute_xpm=NULL, *munmute_xpm=NULL;
  GtkWidget *simul_xpm=NULL, *unsimul_xpm=NULL;
  GtkWidget *mrec_xpm=NULL, *munrec_xpm=NULL;
  GtkWidget *rrec_xpm=NULL, *runrec_xpm=NULL;
  GtkWidget *lrec_xpm=NULL, *lunrec_xpm=NULL;
  GdkPixmap *pixmap=NULL;
  GdkBitmap *mask=NULL, *chan_mask=NULL;
  GtkStyle *style=NULL;
  MixerInfo *info=NULL;

  box = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(box);
  
  /* Create the control label */
  if(is_xpm) {
    style = gtk_widget_get_style(xpmparent);
    pixmap = gdk_pixmap_create_from_xpm(xpmparent->window,
						 &mask,
						 &style->bg[GTK_STATE_NORMAL],
						 name);
    chan_xpm = gtk_pixmap_new(pixmap, mask);
    gtk_widget_show(chan_xpm);
    gtk_table_attach_defaults(GTK_TABLE (table), chan_xpm, 
			      channel, channel + 1,
			      0, 1);
			    
/*     gtk_box_pack_start(GTK_BOX (box), chan_xpm, FALSE, FALSE, 0); */
  }
  else{
    label = gtk_label_new(name);
    gtk_widget_show(label);
/*     gtk_box_pack_start(GTK_BOX (box), label, FALSE, FALSE, 0); */
    gtk_table_attach_defaults(GTK_TABLE (table), label, 
			      channel, channel + 1, 0, 1);
  }

  /********************/
  /* The Widgets *****/

  if(chan_info->caps & SND_MIXER_CINFO_CAP_STEREO) {
    /******************************/
    /* Set up the range controls */
    /* Make the box to hold the range controls */
    box1 = gtk_hbox_new(FALSE, 0);
    gtk_widget_show(box1);
    gtk_table_attach_defaults(GTK_TABLE (table), box1,
			      channel, channel + 1, 1, 2);

    /* Start with the left volume control */
    ladj = gtk_adjustment_new(101.0 - left, 0.0, 101.0, 1.0, 10.0, 0.5);
    lscale = gtk_vscale_new(GTK_ADJUSTMENT (ladj));
    gtk_widget_set_usize(lscale, -1, config.scale);
    gtk_widget_show(lscale);
    gtk_scale_set_draw_value(GTK_SCALE (lscale), FALSE);
    gtk_box_pack_start(GTK_BOX (box1), lscale, TRUE, TRUE, 0);
    /* Store the widget in the channel struct */
    schan->ladj = ladj;
    

    /* Create the right volume control */
    radj = gtk_adjustment_new(101.0 - right, 0.0, 101.0, 1.0, 10.0, 0.5);
    rscale = gtk_vscale_new(GTK_ADJUSTMENT (radj));
    gtk_widget_set_usize(rscale, -1, config.scale);
    gtk_widget_show(rscale);
    gtk_scale_set_draw_value(GTK_SCALE (rscale), FALSE);
    gtk_box_pack_start(GTK_BOX (box1), rscale, TRUE, TRUE, 0);
    /* Store the widget in the channel struct */
    schan->radj = radj;


    /*****************************/
    /* Set up the Mute controls */
    /* Make the box to hold the range controls */
    box1 = gtk_hbox_new(FALSE, 0);
    gtk_widget_show(box1);
    gtk_table_attach_defaults(GTK_TABLE (table), box1,
			      channel, channel + 1, 2, 3);


    /* Only set up the mute if the channel can be muted */
    if(chan_info->caps & SND_MIXER_CINFO_CAP_MUTE ||
       chan_info->caps & SND_MIXER_CINFO_CAP_HWMUTE) {
      /* Set up the left mute button */
      lmute = gtk_toggle_button_new();
      if(config.flags & CONFIG_MUTE_XPM) {
	tmpbox = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(tmpbox);
	gtk_container_add(GTK_CONTAINER(lmute), tmpbox);
	lmute_xpm = gtk_pixmap_new(gdk_pixmap_ref(config.mute_xpm), 
				   config.mute_mask );
	if(data->flags & SND_MIXER_FLG_MUTE_LEFT ||
	   !(config.flags & CONFIG_UNMUTE_XPM))
	  gtk_widget_show(lmute_xpm);
	gtk_box_pack_start(GTK_BOX (tmpbox), lmute_xpm, FALSE, FALSE, 0);

	if(config.flags & CONFIG_UNMUTE_XPM) {
	  lunmute_xpm = gtk_pixmap_new(gdk_pixmap_ref(config.unmute_xpm), 
				       config.unmute_mask);
	  if(!(data->flags & SND_MIXER_FLG_MUTE_LEFT))
	    gtk_widget_show(lunmute_xpm);
	  gtk_box_pack_start(GTK_BOX (tmpbox), lunmute_xpm, FALSE, FALSE, 0);
	}
      }
      else {
	label = gtk_label_new(config.mute);
	gtk_widget_show(label);
	gtk_container_add(GTK_CONTAINER(lmute), label);
      }
      gtk_widget_show(lmute);
      gtk_box_pack_start(GTK_BOX(box1), lmute, TRUE, TRUE, 0);
      /* Set the appropriate initial state */
      if(data->flags & SND_MIXER_FLG_MUTE_LEFT)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (lmute), TRUE);
    }
    /* Store the widget in the channel struct */
    schan->lm = lmute;
    
    
    
    
    /* Only set up the mute if the channel can be muted */
    if(chan_info->caps & SND_MIXER_CINFO_CAP_MUTE ||
       chan_info->caps & SND_MIXER_CINFO_CAP_HWMUTE) {
      /* Set up the left mute button */
      rmute = gtk_toggle_button_new();
      if(config.flags & CONFIG_MUTE_XPM) {
	tmpbox = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(tmpbox);
	gtk_container_add(GTK_CONTAINER(rmute), tmpbox);
	rmute_xpm = gtk_pixmap_new(gdk_pixmap_ref(config.mute_xpm), 
				   config.mute_mask );
	if(data->flags & SND_MIXER_FLG_MUTE_RIGHT ||
	   !(config.flags & CONFIG_UNMUTE_XPM))
	  gtk_widget_show(rmute_xpm);
	gtk_box_pack_start(GTK_BOX (tmpbox), rmute_xpm, FALSE, FALSE, 0);
	
	if(config.flags & CONFIG_UNMUTE_XPM) {
	  runmute_xpm = gtk_pixmap_new(gdk_pixmap_ref(config.unmute_xpm), 
				       config.unmute_mask);
	  if(!(data->flags & SND_MIXER_FLG_MUTE_RIGHT))
	    gtk_widget_show(runmute_xpm);
	  gtk_box_pack_start(GTK_BOX (tmpbox), runmute_xpm, FALSE, FALSE, 0);
	}
      }
      else {
	label = gtk_label_new(config.mute);
	gtk_widget_show(label);
	gtk_container_add(GTK_CONTAINER(rmute), label);
      }

      gtk_widget_show(rmute);
      gtk_box_pack_start(GTK_BOX(box1), rmute, TRUE, TRUE, 0);
      /* Set the appropriate initial state */
      if(data->flags & SND_MIXER_FLG_MUTE_RIGHT)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (rmute), TRUE);
    }
    /* Store the widget in the channel struct */
    schan->rm = rmute;
    
    
    /**********************/
    /* The Callbacks *****/
    
    /* The left */
    /* The Scale callback */
    info = calloc(1, sizeof(MixerInfo));
    info->mixer = mixer;
    info->channel = channel;
    info->other = (GtkWidget *)radj;
    info->chan = schan;
    info->flags &= 0;
    info->flags |= CHANNEL_LEFT;
    gtk_signal_connect(GTK_OBJECT (ladj), "value_changed",
		       (GtkSignalFunc) volume_adjust, info);
    
    /* Only set up the mute if the channel can be muted */
    if(chan_info->caps & SND_MIXER_CINFO_CAP_MUTE ||
       chan_info->caps & SND_MIXER_CINFO_CAP_HWMUTE) {
      /* The Mute callback */
      info = calloc(1, sizeof(MixerInfo));
      info->mixer = mixer;
      info->channel = channel;
      info->other = rmute;
      info->chan = schan;
      info->mute = lmute_xpm;
      info->unmute = lunmute_xpm;
      info->flags &= 0;
      info->flags |= CHANNEL_MUTE_LEFT;
      gtk_signal_connect(GTK_OBJECT (lmute), "toggled",
			 (GtkSignalFunc) volume_adjust, info);
    }
    
    
    /* The Right */
    /* Set up the Right callback */  
    info = calloc(1, sizeof(MixerInfo));
    info->mixer = mixer;
    info->channel = channel;
    info->other = (GtkWidget *)ladj;
    info->chan = schan;
    info->flags &= 0;
    info->flags |= CHANNEL_RIGHT;
    gtk_signal_connect(GTK_OBJECT (radj), "value_changed",
		       (GtkSignalFunc) volume_adjust, info);
    /* Set up the Mute callback */
    /* Only set up the mute if the channel can be muted */
    if(chan_info->caps & SND_MIXER_CINFO_CAP_MUTE ||
       chan_info->caps & SND_MIXER_CINFO_CAP_HWMUTE) {
      info = calloc(1, sizeof(MixerInfo));
      info->mixer = mixer;
      info->channel = channel;
      info->other = lmute;
      info->chan = schan;
      info->mute = rmute_xpm;
      info->unmute = runmute_xpm;
      info->flags &= 0;
      info->flags |= CHANNEL_MUTE_RIGHT;
      gtk_signal_connect(GTK_OBJECT (rmute), "toggled",
			 (GtkSignalFunc) volume_adjust, info);
    }
    
  }
  /* If the channel isn't stero */
  else {
    madj = gtk_adjustment_new(101.0 - left, 0.0, 101.0, 1.0, 10.0, 0.5);
    lscale = gtk_vscale_new(GTK_ADJUSTMENT (madj));
    gtk_scale_set_draw_value(GTK_SCALE (lscale), FALSE);
    gtk_widget_set_usize(lscale, -1, config.scale);
    gtk_widget_show(lscale);
    gtk_scale_set_draw_value(GTK_SCALE (lscale), FALSE);
/*     gtk_box_pack_start(GTK_BOX (box), lscale, FALSE, FALSE, 0); */
    gtk_table_attach_defaults(GTK_TABLE (table), lscale,
			      channel, channel + 1, 1, 2);
    /* Store the widget in the channel struct */
    schan->madj = madj;

    /* Set up the volume callback */
    info = calloc(1, sizeof(MixerInfo));
    info->mixer = mixer;
    info->channel = channel;
    info->other = NULL;
    info->chan = schan;
    info->flags &= 0;
    info->flags |= CHANNEL_MONO;
    gtk_signal_connect(GTK_OBJECT (madj), "value_changed",
		       (GtkSignalFunc) volume_adjust, info);


    /* Only set up the mute if the channel can be muted */
    if(chan_info->caps & SND_MIXER_CINFO_CAP_MUTE ||
       chan_info->caps & SND_MIXER_CINFO_CAP_HWMUTE) {
      /* Set up the right mute button */
      mmute = gtk_toggle_button_new();
      if(config.flags & CONFIG_MUTE_XPM) {
	tmpbox = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(tmpbox);
	gtk_container_add(GTK_CONTAINER(mmute), tmpbox);
	mmute_xpm = gtk_pixmap_new(gdk_pixmap_ref(config.mute_xpm), 
				   config.mute_mask );
	if(data->flags & SND_MIXER_FLG_MUTE_LEFT ||
	   !(config.flags & CONFIG_UNMUTE_XPM))
	  gtk_widget_show(mmute_xpm);
	gtk_box_pack_start(GTK_BOX (tmpbox), mmute_xpm, FALSE, FALSE, 0);

	if(config.flags & CONFIG_UNMUTE_XPM) {
	  munmute_xpm = gtk_pixmap_new(gdk_pixmap_ref(config.unmute_xpm), 
				       config.unmute_mask);
	  if(!(data->flags & SND_MIXER_FLG_MUTE_LEFT))
	    gtk_widget_show(munmute_xpm);
	  gtk_box_pack_start(GTK_BOX (tmpbox), munmute_xpm, FALSE, FALSE, 0);
	}
      }
      else {
	label = gtk_label_new(config.mute);
	gtk_widget_show(label);
	gtk_container_add(GTK_CONTAINER(mmute), label);
      }


      gtk_widget_show(mmute);
/*       gtk_box_pack_start(GTK_BOX(box), mmute, FALSE, FALSE, 0); */
      gtk_table_attach_defaults(GTK_TABLE (table), mmute,
				channel, channel + 1, 2, 3); 
      /* Set the appropriate initial state */
      if(data->flags & SND_MIXER_FLG_MUTE_LEFT)
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (mmute), TRUE);
    }
    /* Store the widget in the channel struct */
    schan->mm = mmute;

    /* Set up the mute callback */
    info = calloc(1, sizeof(MixerInfo));
    info->mixer = mixer;
    info->channel = channel;
    info->other = NULL;
    info->chan = schan;
    info->mute = mmute_xpm;
    info->unmute = munmute_xpm;
    info->flags &= 0;
    info->flags |= CHANNEL_MUTE_MONO;
    gtk_signal_connect(GTK_OBJECT (mmute), "toggled",
		       (GtkSignalFunc) volume_adjust, info);

  }


  /***************************************/
  /* Create the simultaneous button *****/
  if(chan_info->caps & SND_MIXER_CINFO_CAP_STEREO) {
    button = gtk_toggle_button_new();
    if(config.flags & CONFIG_SIMUL_XPM) {
      tmpbox = gtk_vbox_new(FALSE, 0);
      gtk_widget_show(tmpbox);
      gtk_container_add(GTK_CONTAINER(button), tmpbox);
      simul_xpm = gtk_pixmap_new(gdk_pixmap_ref(config.simul_xpm), 
				 config.simul_mask );
      gtk_widget_show(simul_xpm);
      gtk_box_pack_start(GTK_BOX (tmpbox), simul_xpm, FALSE, FALSE, 0);
      
      if(config.flags & CONFIG_UNSIMUL_XPM) {
	unsimul_xpm = gtk_pixmap_new(gdk_pixmap_ref(config.unsimul_xpm), 
				     config.unsimul_mask);
	gtk_box_pack_start(GTK_BOX (tmpbox), unsimul_xpm, FALSE, FALSE, 0);
      }
    }
    else {
      label = gtk_label_new(config.simul);
      gtk_widget_show(label);
      gtk_container_add(GTK_CONTAINER(button), label);
    }
    
    gtk_widget_show(button);
    /* gtk_box_pack_start(GTK_BOX (box), button, FALSE, FALSE, 0); */
    gtk_table_attach_defaults(GTK_TABLE (table), button,
			      channel, channel + 1, 3, 4);
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
    /* Set up the callback */
    info = calloc(1, sizeof(MixerInfo));
    info->mixer = mixer;
    info->channel = channel;
    info->chan = schan;
    info->mute = simul_xpm;
    info->unmute = unsimul_xpm;
    info->flags &= 0;
    info->flags |= CHANNEL_SIMULTANEOUS;
    gtk_signal_connect(GTK_OBJECT (button), "toggled",
		       (GtkSignalFunc) volume_adjust, info);
    /* And let's not forget to set the simultaneous flag */
    schan->flags |= CHANNEL_SIMULTANEOUS;
  }

  /*********************************/
  /* Create the Record button *****/
  if(chan_info->caps & SND_MIXER_CINFO_CAP_RECORD)
    if(chan_info->caps & SND_MIXER_CINFO_CAP_JOINRECORD || 
       !(chan_info->caps & SND_MIXER_CINFO_CAP_STEREO)) {
    mrec = gtk_toggle_button_new();
    if(config.flags & CONFIG_REC_XPM) {
      tmpbox = gtk_vbox_new(FALSE, 0);
      gtk_widget_show(tmpbox);
      gtk_container_add(GTK_CONTAINER(mrec), tmpbox);
      mrec_xpm = gtk_pixmap_new(gdk_pixmap_ref(config.rec_xpm), 
				 config.rec_mask );
      gtk_box_pack_start(GTK_BOX (tmpbox), mrec_xpm, FALSE, FALSE, 0);
      if(data->flags & SND_MIXER_FLG_RECORD || 
	 !(config.flags & CONFIG_UNREC_XPM))
	gtk_widget_show(mrec_xpm);
      
      if(config.flags & CONFIG_UNREC_XPM) {
	munrec_xpm = gtk_pixmap_new(gdk_pixmap_ref(config.unrec_xpm), 
				     config.unrec_mask);
	gtk_box_pack_start(GTK_BOX (tmpbox), munrec_xpm, FALSE, FALSE, 0);
	if(!(data->flags & SND_MIXER_FLG_RECORD))
	  gtk_widget_show(munrec_xpm);
      }
    }
    else {
      label = gtk_label_new(config.rec);
      gtk_widget_show(label);
      gtk_container_add(GTK_CONTAINER(mrec), label);
    }
    gtk_widget_show(mrec);
/*     gtk_box_pack_start(GTK_BOX (box), button, FALSE, FALSE, 0); */
    gtk_table_attach_defaults(GTK_TABLE (table), mrec,
			      channel, channel + 1, 4, 5);
    /* Store the widget in the channel struct */
    schan->rec = mrec;
    
    /* Set the appropriate initial state */
    if(data->flags & SND_MIXER_FLG_RECORD)
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (mrec), TRUE);

    /* Set up the callback */
    info = calloc(1, sizeof(MixerInfo));
    info->mixer = mixer;
    info->channel = channel;
    info->chan = schan;
    info->mute = mrec_xpm;
    info->unmute = munrec_xpm;
    info->flags &= 0;
    info->flags |= CHANNEL_RECORD;
    gtk_signal_connect(GTK_OBJECT (mrec), "toggled",
		       (GtkSignalFunc) volume_adjust, info);
  }
  /* Stereo Record */
  else { 
    /* Make the container box up */
    box2 = gtk_hbox_new(FALSE, 0);
    gtk_widget_show(box2);
    gtk_table_attach_defaults(GTK_TABLE (table), box2, 
			      channel, channel + 1, 4, 5);
    
    /******************/
    /* The left side */
    lrec = gtk_toggle_button_new();
    if(config.flags & CONFIG_REC_XPM) {
      tmpbox = gtk_vbox_new(FALSE, 0);
      gtk_widget_show(tmpbox);
      gtk_container_add(GTK_CONTAINER(lrec), tmpbox);
      lrec_xpm = gtk_pixmap_new(gdk_pixmap_ref(config.rec_xpm), 
				 config.rec_mask );
      gtk_box_pack_start(GTK_BOX (tmpbox), lrec_xpm, FALSE, FALSE, 0);
      if(data->flags & SND_MIXER_FLG_RECORD_LEFT || 
	 !(config.flags & CONFIG_UNREC_XPM))
	gtk_widget_show(lrec_xpm);
      
      if(config.flags & CONFIG_UNREC_XPM) {
	lunrec_xpm = gtk_pixmap_new(gdk_pixmap_ref(config.unrec_xpm), 
				     config.unrec_mask);
	gtk_box_pack_start(GTK_BOX (tmpbox), lunrec_xpm, FALSE, FALSE, 0);
	if(!(data->flags & SND_MIXER_FLG_RECORD_LEFT))
	  gtk_widget_show(lunrec_xpm);
      }
    }
    else {
      label = gtk_label_new(config.rec);
      gtk_widget_show(label);
      gtk_container_add(GTK_CONTAINER(lrec), label);
    }
    gtk_widget_show(lrec);
    gtk_box_pack_start(GTK_BOX (box2), lrec, TRUE, TRUE, 0);
    /* Store the widget in the channel struct */
    schan->lrec = lrec;
    
    /* Set the appropriate initial state */
    if(data->flags & SND_MIXER_FLG_RECORD_LEFT)
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (lrec), TRUE);


    /******************/
    /* The right side */
    rrec = gtk_toggle_button_new();
    if(config.flags & CONFIG_REC_XPM) {
      tmpbox = gtk_vbox_new(FALSE, 0);
      gtk_widget_show(tmpbox);
      gtk_container_add(GTK_CONTAINER(rrec), tmpbox);
      rrec_xpm = gtk_pixmap_new(gdk_pixmap_ref(config.rec_xpm), 
				 config.rec_mask );
      gtk_box_pack_start(GTK_BOX (tmpbox), rrec_xpm, FALSE, FALSE, 0);
      if(data->flags & SND_MIXER_FLG_RECORD_RIGHT || 
	 !(config.flags & CONFIG_UNREC_XPM))
	gtk_widget_show(rrec_xpm);
      
      if(config.flags & CONFIG_UNREC_XPM) {
	runrec_xpm = gtk_pixmap_new(gdk_pixmap_ref(config.unrec_xpm), 
				     config.unrec_mask);
	gtk_box_pack_start(GTK_BOX (tmpbox), runrec_xpm, FALSE, FALSE, 0);
	if(!(data->flags & SND_MIXER_FLG_RECORD_RIGHT))
	  gtk_widget_show(runrec_xpm);
      }
    }
    else {
      label = gtk_label_new(config.rec);
      gtk_widget_show(label);
      gtk_container_add(GTK_CONTAINER(rrec), label);
    }
    gtk_widget_show(rrec);
    gtk_box_pack_start(GTK_BOX (box2), rrec, TRUE, TRUE, 0);
    /* Store the widget in the channel struct */
    schan->rrec = rrec;
    
    /* Set the appropriate initial state */
    if(data->flags & SND_MIXER_FLG_RECORD_RIGHT)
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (rrec), TRUE);



    /* The Left Record callback */
    info = calloc(1, sizeof(MixerInfo));
    info->mixer = mixer;
    info->channel = channel;
    info->other = rrec;
    info->chan = schan;
    info->mute = lmute_xpm;
    info->unmute = lunrec_xpm;
    info->flags &= 0;
    info->flags |= CHANNEL_RECORD_LEFT;
    gtk_signal_connect(GTK_OBJECT (lrec), "toggled",
		       (GtkSignalFunc) volume_adjust, info);
    
    /* The Right Record Callback */
    info = calloc(1, sizeof(MixerInfo));
    info->mixer = mixer;
    info->channel = channel;
    info->other = lrec;
    info->chan = schan;
    info->mute = rrec_xpm;
    info->unmute = runrec_xpm;
    info->flags &= 0;
    info->flags |= CHANNEL_RECORD_RIGHT;
    gtk_signal_connect(GTK_OBJECT (rrec), "toggled",
		       (GtkSignalFunc) volume_adjust, info);
    
  }



  return box;
}
